import { APIGatewayProxyEvent, APIGatewayProxyResult, Context } from 'aws-lambda';
import { Request } from './types';
import { Static } from '@sinclair/typebox';
declare type AnyRequest<TBody> = {
    pathParams: any;
    queryParams: any;
    body: TBody;
};
declare type RouteHandlerDefinition = {
    method: string;
    url: string;
    body?: any;
    handler: (req: AnyRequest<any>, apiParams: {
        originalEvent: APIGatewayProxyEvent;
        context: Context;
    }) => Promise<APIGatewayProxyResult>;
};
declare type HttpMethod<R, M extends HTTPMethod> = <A extends string, B = never>(path: A, body?: M extends HTTPRead ? never : B) => (handler: (req: Request<A, Static<B>>, originalEvent: {
    originalEvent: APIGatewayProxyEvent;
    context: Context;
}) => Promise<APIGatewayProxyResult>) => Router<R>;
export declare type Router<R> = R & {
    get: HttpMethod<R, 'get'>;
    head: HttpMethod<R, 'head'>;
    options: HttpMethod<R, 'options'>;
    post: HttpMethod<R, 'post'>;
    put: HttpMethod<R, 'put'>;
    delete: HttpMethod<R, 'delete'>;
};
export declare type RouteHandlers = {
    handlers: readonly RouteHandlerDefinition[];
};
declare type HTTPRead = 'get' | 'options' | 'head';
declare type HTTPWrite = 'post' | 'put' | 'delete';
declare type HTTPMethod = HTTPRead | HTTPWrite;
export declare const Router: (handlers?: readonly RouteHandlerDefinition[]) => Router<RouteHandlers>;
export {};
