import { PathParamParser, PathParamParsers } from "./path-param-parser";
export declare type Logger = {
    info: (msg: string, metadata?: object) => void;
};
export declare type CorsConfig = {
    allowHeaders: "*" | string[];
    allowOrigin: string | string[];
    allowMethods: "*" | string[];
    allowCredentials: boolean;
};
export declare type RouterConfig = {
    logger?: Logger;
    corsConfig?: true | CorsConfig;
};
declare type _<T> = T;
export declare type Merge<T> = _<{
    [k in keyof T]: T[k];
}>;
export declare type Trim<T> = T extends `/${infer Rest}${"/" | ""}` ? Trim<Rest> : T;
declare type TypeOfParser<T> = T extends PathParamParser<infer A> ? A : never;
declare type ParserType<T extends keyof PathParamParsers> = TypeOfParser<PathParamParsers[T]>;
declare type PathParam<S extends string> = S extends `${infer Var}:${infer VarType}` ? VarType extends keyof PathParamParsers ? {
    readonly [key in Var]: ParserType<VarType>;
} : never : S extends `${infer Var}` ? {
    readonly [key in Var]: string;
} : never;
declare type PathParams<A extends string, Seed = {}> = A extends `{${infer AA}}${infer Tail}` ? Merge<Seed & PathParam<AA> & PathParams<Tail>> : A extends `${infer _}{${infer AA}}${infer Tail}` ? Merge<Seed & PathParam<AA> & PathParams<Tail>> : A extends `${infer _}{${infer AA}}` ? Merge<Seed & PathParam<AA>> : Seed;
declare type UrlParam<S extends string, P = string> = S extends `${infer Var}:${infer VarType}` ? VarType extends keyof PathParamParsers ? UrlParam<Var, ParserType<VarType>> : never : S extends `${infer Var}?` ? {
    readonly [key in Var]?: P;
} : S extends `${infer Var}` ? {
    readonly [key in Var]: P;
} : never;
declare type QueryParams<A extends string, Seed = {}> = A extends `{${infer AA}}${infer Tail}` ? Merge<Seed & UrlParam<AA> & QueryParams<Tail>> : Seed;
export declare type Request<Url extends string, Body = unknown> = {
    pathParams: Url extends `${infer P}?${infer _}` ? PathParams<P> : PathParams<Url>;
    queryParams: Url extends `${infer _}?${infer Q}` ? QueryParams<Q> : never;
    body: Body;
};
export {};
